@@ -25,20 +25,15 @@
 
 #include <libxml/xmlreader.h>
 
-#include "gtksourcelanguagesmanager.h"
-
-#include "gtksourcelanguage-private.h"
 #include "gtksourceview-i18n.h"
 
+#include "gtksourcelanguagesmanager.h"
+#include "gtksourcelanguage-private.h"
 #include "gtksourcelanguage.h"
 #include "gtksourcetag.h"
 
 #define DEFAULT_GCONF_BASE_DIR		"/apps/gtksourceview"
 
-#define DEFAULT_LANGUAGE_DIR		DATADIR "/gtksourceview-1.0/language-specs"
-#define USER_LANGUAGE_DIR		"gtksourceview-1.0/language-specs"
-#define USER_CONFIG_BASE_DIR	".gnome2"
-
 enum {
 	PROP_0,
 	PROP_LANG_SPECS_DIRS
@@ -177,7 +172,13 @@ gtk_source_languages_manager_instance_init (GtkSourceLanguagesManager *lm)
 	lm->priv = g_new0 (GtkSourceLanguagesManagerPrivate, 1);	
 }
 
-
+/**
+ * gtk_source_languages_manager_new:
+ *
+ * Creates a new language manager.
+ *
+ * Return value: a #GtkSourceLanguagesManager.
+ **/
 GtkSourceLanguagesManager *
 gtk_source_languages_manager_new (void)
 {
@@ -208,23 +209,42 @@ gtk_source_languages_manager_finalize (GObject *object)
 	(* G_OBJECT_CLASS (parent_class)->finalize) (object);
 }
 
-static void 
-gtk_source_languages_manager_set_specs_dirs (GtkSourceLanguagesManager	*lm,
-					     const GSList		*dirs)
+#define SOURCEVIEW_DIR		"gtksourceview-1.0"
+#define LANGUAGE_DIR		"language-specs"
+#define USER_CONFIG_BASE_DIR	".gnome2"
+
+static void
+gtk_source_languages_manager_set_specs_dirs (GtkSourceLanguagesManager *lm,
+					     const GSList              *dirs)
 {
 	g_return_if_fail (GTK_IS_SOURCE_LANGUAGES_MANAGER (lm));
 	g_return_if_fail (lm->priv->language_specs_directories == NULL);
-			
+
 	if (dirs == NULL)
 	{
-		lm->priv->language_specs_directories =
-			g_slist_prepend (lm->priv->language_specs_directories,
-					g_strdup (DEFAULT_LANGUAGE_DIR));
+		const gchar * const *xdg_dirs;
+		gint i;
+
+		/* fetch specs in XDG_DATA_DIRS */
+		xdg_dirs = g_get_system_data_dirs ();
+		for (i = G_N_ELEMENTS (xdg_dirs); i >= 0; --i)
+		{
+			lm->priv->language_specs_directories =
+				g_slist_prepend (lm->priv->language_specs_directories,
+						 g_build_filename (xdg_dirs[i],
+								   SOURCEVIEW_DIR,
+								   LANGUAGE_DIR, 
+								   NULL));
+		}
+
+		/* prepend the dir in ~ so that we look there first */
 		lm->priv->language_specs_directories = 
 			g_slist_prepend (lm->priv->language_specs_directories,
-					g_build_filename (g_get_home_dir(), 
-						USER_CONFIG_BASE_DIR, USER_LANGUAGE_DIR, 
-						NULL));
+					 g_build_filename (g_get_home_dir(),
+							   USER_CONFIG_BASE_DIR,
+							   SOURCEVIEW_DIR,
+							   LANGUAGE_DIR, 
+							   NULL));
 
 		return;
 	}
@@ -239,6 +259,14 @@ gtk_source_languages_manager_set_specs_dirs (GtkSourceLanguagesManager	*lm,
 	}
 }
 
+/**
+ * gtk_source_languages_manager_get_lang_files_dirs:
+ * @lm: a #GtkSourceLanguagesManager.
+ * 
+ * Gets a list of language files directories for the given language manager.
+ *
+ * Return value: a list of language files directories (as strings).
+ **/
 const GSList *
 gtk_source_languages_manager_get_lang_files_dirs (GtkSourceLanguagesManager *lm)
 {
@@ -247,6 +275,16 @@ gtk_source_languages_manager_get_lang_files_dirs (GtkSourceLanguagesManager *lm)
 	return lm->priv->language_specs_directories;
 }
 
+/**
+ * gtk_source_languages_manager_get_available_languages:
+ * @lm: a #GtkSourceLanguagesManager.
+ * 
+ * Gets a list of available languages for the given language manager.
+ * This function returns a pointer to a internal list, so there is no need to
+ * free it after usage.
+ *
+ * Return value: a list of #GtkSourceLanguage.
+ **/
 const GSList *
 gtk_source_languages_manager_get_available_languages (GtkSourceLanguagesManager *lm)
 {
@@ -290,6 +328,17 @@ gtk_source_languages_manager_get_available_languages (GtkSourceLanguagesManager
 	return lm->priv->available_languages;
 }
 
+/**
+ * gtk_source_languages_manager_get_language_from_mime_type:
+ * @lm: a #GtkSourceLanguagesManager.
+ * @mime_type: a mime type.
+ * 
+ * Gets the #GtkSourceLanguage which is associated with the given @mime_type
+ * in the language manager.
+ *
+ * Return value: a #GtkSourceLanguage, or %NULL if there is no language
+ * associated with the given @mime_type.
+ **/
 GtkSourceLanguage *
 gtk_source_languages_manager_get_language_from_mime_type (GtkSourceLanguagesManager 	*lm,
 							  const gchar 			*mime_type)
